"""command-line interface for eatlocal

"""

import sys
from collections import namedtuple
from pathlib import Path

import typer
from rich.status import Status

from . import __version__
from .constants import PASSWORD, USERNAME
from .eatlocal import display_bite, download_bite, extract_bite, submit_bite

cli = typer.Typer(add_completion=False)


def report_version(display: bool) -> None:
    """Print version and exit."""
    if display:
        print(f"{Path(sys.argv[0]).name} {__version__}")
        raise typer.Exit()


GlobalOptions = namedtuple("GlobalOptions", "creds")


@cli.callback()
def global_options(
    ctx: typer.Context,
    version: bool = typer.Option(
        False,
        "--version",
        "-v",
        is_flag=True,
        is_eager=True,
        callback=report_version,
    ),
):
    """Download, extract, and submit PyBites code challenges."""

    ctx.obj = GlobalOptions((USERNAME, PASSWORD))


@cli.command(name="download")
def download_subcommand(
    ctx: typer.Context,
    bite_number: int,
    cleanup: bool = typer.Option(
        False,
        "--cleanup",
        "-C",
        is_flag=True,
        help="Remove downloaded bite archive file.",
    ),
    verbose: bool = typer.Option(
        False,
        "--verbose",
        "-V",
        is_flag=True,
        help="Print each step as it happens.",
    ),
    force: bool = typer.Option(
        False,
        "--force",
        "-F",
        is_flag=True,
        help="Overwrite bite directory with a fresh version.",
    ),
) -> None:
    """Download and extract bite code from Codechalleng.es.

    The bites are downloaded in a zip archive file and unzipped
    in the current directory. If the `cleanup` option is present
    the archive is deleted after extraction.
    """
    with Status(f"Downloading Bite {bite_number}") as status:
        download_bite(bite_number, *ctx.obj.creds, cache_path="cache", verbose=verbose)
        status.update(f"Extracting Bite {bite_number}")
        extract_bite(bite_number, cleanup=cleanup, cache_path="cache", force=force)


@cli.command(name="submit")
def submit_subcommand(
    ctx: typer.Context,
    bite_number: int,
    verbose: bool = typer.Option(
        False,
        "--verbose",
        "-V",
        is_flag=True,
        help="Print each step as it happens",
    ),
) -> None:
    """Submit a bite back to Codechalleng.es."""

    with Status(f"Submitting Bite {bite_number}"):
        submit_bite(bite_number, *ctx.obj.creds, verbose=verbose)


@cli.command(name="display")
def read_subcommand(
    ctx: typer.Context,
    bite_number: int,
    bite_path: Path = None,
    theme: str = typer.Option(
        "material",
        "--theme",
        "-t",
        is_flag=True,
        help="Choose syntax highlighting for code.",
    ),
) -> None:
    """Read a bite directly in the terminal."""

    display_bite(bite_number, theme=theme)


if __name__ == "__main__":
    cli()



""" download and submit bites
"""


import webbrowser
from pathlib import Path
from time import sleep
from typing import Union
from zipfile import ZipFile, is_zipfile

from bs4 import BeautifulSoup
from getkey import getkey, keys
from git import Repo
from rich import print
from rich.layout import Layout
from rich.live import Live
from rich.panel import Panel
from rich.syntax import Syntax
from rich.traceback import install
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By

from .constants import BITE_REPO, BITE_URL, BITE_ZIPFILE, SUBMIT_URL
from .pydriver import driver_setup, pybites_login

install(show_locals=True)


def find_cached_archive(bite_number: int, path: Union[str, Path] = None) -> Path:
    """Return a Path for a PyBites bite zip archive in the given `path`.

    :bite_number: int
    :path: optional Path, resolved path defaults to current directory.
    :return: Path

    Raises:
    - FileNotFoundError if archive not found in the target path.
    """

    path = Path(path or Path.cwd())

    filename = BITE_ZIPFILE.format(bite_number=bite_number)

    try:
        archive = list(path.rglob(filename))[0]
    except IndexError:
        raise FileNotFoundError(filename) from None

    return archive.resolve()


def download_bite(
    bite_number: int,
    username: str,
    password: str,
    delay: float = 1.5,
    cache_path: Path = None,
    verbose: bool = False,
) -> None:
    """Download bite ZIP archive file from the platform to the current directory.

    :bite_number: int The number of the bite to download.
    :username: str
    :password: str
    :delay: float Time in seconds to pause between operations
    :cache_path: Path for cached ZIP archive files, defaults to current directory
    :returns: None
    """

    try:
        path = find_cached_archive(bite_number, path=cache_path)
        print(f"Bite {bite_number} found: @ {path}")
        return
    except FileNotFoundError:
        pass

    cache_path = Path(cache_path or Path.cwd()).resolve()
    cache_path.mkdir(mode=0o755, parents=True, exist_ok=True)

    if verbose:
        print(f"Retrieving bite {bite_number}...")
    sleep(delay)

    driver = driver_setup(cache_path)
    pybites_login(driver, username, password, verbose=verbose)
    driver.get(BITE_URL.format(bite_number=bite_number))
    sleep(delay)

    try:
        bite_ziparchive = find_cached_archive(bite_number, path=cache_path)
    except FileNotFoundError:
        print(f"[yellow]Bite {bite_number} was not downloaded.[/yellow]")
        return

    if not is_zipfile(bite_ziparchive):
        print(f"[yellow]Bite {bite_number} is not a valid archive file.[/yellow]")
        return

    if verbose:
        print(f"Bite {bite_number} successully downloaded: {bite_ziparchive}")


def extract_bite(
    bite_number: int,
    dest_path: Path = None,
    cleanup: bool = False,
    cache_path: Path = None,
    force: bool = False,
) -> None:
    """Extracts all the required files into a new directory
    named by the bite number.

    :bite_number: int The number of the bite you want to extract.
    :cleanup: bool if False removes the downloaded zipfile.
    :cache_path: Path to search for ZIp archive, defaults to current directory.
    :force: bool if True overwrites the directory for the bite_number.
    :returns: None
    """

    try:
        bite = find_cached_archive(bite_number, path=cache_path)
    except FileNotFoundError as error:
        print(f"[yellow]Missing ZIP archive for bite {bite_number}: {error}[/yellow]")
        return

    dest_path = Path(dest_path or Path.cwd()).resolve() / str(bite_number)

    if dest_path.is_dir() and not force:
        print(f"[yellow]There already exists a directory for bite {bite_number}. Use the --force flag to overwite.[/yellow]")
        return

    else:
        with ZipFile(bite, "r") as zipfile:
            zipfile.extractall(dest_path)

        print(f"Extracted bite {bite_number} @ {dest_path}")

        if cleanup:
            print(f"Cleaning up bite {bite_number} archive: {bite}")
            bite.unlink()


def submit_bite(
    bite_number: int,
    username: str,
    password: str,
    delay: float = 1.0,
    verbose: bool = False,
) -> None:
    """Submits bite by pushing to GitHub and then opens a browser for the
       bite page.

    :bite_number: int The number of the bite to submit.
    :username: str
    :password: str
    :delay: float time in seconds to pause between operations.
    :returns: None

    """

    repo = Repo(BITE_REPO)

    try:
        repo.index.add(str(bite_number))
    except FileNotFoundError:
        print(f"[yellow]Seems like there is no bite {bite_number} to submit. Did you mean to submit a different bite?\n[/yellow]")
        return

    repo.index.commit(f"submission Bite {bite_number} @ codechalleng.es")
    repo.remotes.origin.push().raise_if_error()
    if verbose:
        print(f"\nPushed bite {bite_number} to github")

    driver = driver_setup()

    pybites_login(driver, username, password, verbose=verbose)

    bite_url = SUBMIT_URL.format(bite_number=bite_number)

    driver.get(bite_url)
    sleep(delay)

    buttons = {
        "githubDropdown": "Downloading code from GitHub.",
        "ghpull": "",
        "save": f"Submitting bite {bite_number}.",
    }

    for button_name, message in buttons.items():
        if message:
            if verbose:
                print(message)
        try:
            button = driver.find_element(By.ID, button_name)
        except NoSuchElementException:
            print("[yellow]Looks like you've already completed this bite![/yellow]")
            break

        button.click()
        sleep(delay)

    webbrowser.open(bite_url)


def quit_display():
    """Quit the display"""
    quit_keys = ["q", "Q", keys.ESC]
    while True:
        key = getkey()
        if key in quit_keys:
            break


def display_bite(
    bite_number: int,
    bite_path: Path = BITE_REPO,
    theme: str = "material",
) -> None:
    """Display the instructions provided in bite.html and display source code.

    :bite_number: int The number of the bite you want to read
    :returns: None

    """

    path = Path(bite_path or Path.cwd()).resolve() / str(bite_number)

    html_file = path / list(path.glob("*.html"))[0]

    for file in path.iterdir():
        if str(file).endswith(".py") and not str(file.parts[-1]).startswith("test_"):
            python_file = file

    with open(html_file, "r") as bite_html:
        soup = BeautifulSoup(bite_html, "html.parser")
        instructions = soup.text

    with open(path / python_file, "r") as code_file:
        code = Syntax(
            code_file.read(),
            "python",
            theme=theme,
            background_color="default",
        )

    layout = Layout()
    layout.split(
        Layout(name="header", size=3),
        Layout(name="main", ratio=1),
    )
    layout["main"].split_row(
        Layout(name="directions"),
        Layout(name="code"),
    )

    layout["header"].update(
        Panel(f"Displaying Bite {bite_number} at {html_file}", title="eatlocal")
    )
    layout["main"]["directions"].update(Panel(instructions, title="Directions"))
    layout["main"]["code"].update(Panel(code, title="Code"))

    with Live(layout, screen=True):
        quit_display()



import os
from dotenv import load_dotenv

load_dotenv()

USERNAME = os.environ.get("PYBITES_USERNAME", None)
PASSWORD = os.environ.get("PYBITES_PASSWORD", None)
BITE_REPO = os.environ.get("PYBITES_REPO", None)

BITE_URL = "https://codechalleng.es/bites/api/eatlocal/{bite_number}"
LOGIN_URL = "https://codechalleng.es/login"
SUBMIT_URL = "https://codechalleng.es/bites/{bite_number}"
BITE_ZIPFILE = "pybites_bite{bite_number}.zip"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['eatlocal']

package_data = \
{'': ['*']}

install_requires = \
['GitPython>=3.1.26,<4.0.0',
 'beautifulsoup4>=4.10.0,<5.0.0',
 'getkey>=0.6.5,<0.7.0',
 'python-dotenv>=0.19.2,<0.20.0',
 'rich>=11.1.0,<12.0.0',
 'selenium>=4.1.0,<5.0.0',
 'typer>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['eatlocal = eatlocal.__main__:cli']}

setup_kwargs = {
    'name': 'eatlocal',
    'version': '0.7.5',
    'description': 'eatlocal helps the user solve PyBite code challenges on their local machine',
    'long_description': '# eatlocal\n\nEatlocal helps the user solve [PyBites](https://codechalleng.es) code challenges locally. This cli tool allows you to download, unzip, and organize bites according to the expected structure from the directions on the PyBites website. Once you have solved the bite you can use eatlocal to submit and it will open a bowser tab at the correct location.\n\n## DEMOS\n\n### Download Bites\n\n![gif of download command](./docs/demos/download.gif) \n\n### Display Bites\n\n![gif of display command](./docs/demos/display.gif) \n\n### Submit Bites\n\n![gif of submit command](./docs/demos/submit.gif) \n\n## Table of Contents\n\n+ [Usage](#Usage)\n+ [Installation](#Installation)\n\t+ [macOS/Linux](#macoslinux)\n\t+ [Windows](#Windows)\n+ [Setup](#Setup)\n\t+ [Install Chrome and Chromedriver](#Install-Chrome-and-Chromedriver)\n\t\t+ [macOS](#macOS)\n\t\t+ [Linux](#Linux)\n\t\t+ [Windows](#Windows-1)\n\t+ [PyBites Credentials and Local Repo](#PyBites-Credentials-and-Local-Repo)\n\t\t+ [macOS/Linux](#macoslinux-1)\n\t\t+ [Windows](#Windows-2)\n\n\n## Usage\n\nNavigate to your local PyBites repo.\n\nDownload and extract bites:\n\n```bash\n# use -V, --verbose to print what\'s happening\neatlocal download <bite number>\n```\n\nDisplay bites in the terminal:\n\n```bash\n# change the theme with -t <theme name>\neatlocal display <bite number>\n\n\nSubmit bites:\n\n```bash\n# use -V, --verbose to print what\'s happening\neatlocal submit <bite number>\n\n```\n\n## Installation\n\n### macOS/Linux\n\n```bash\npip3 install eatlocal\n```\n### Windows\n\n```bash\npip install eatlocal\n```\n\n## Setup\n\n1. Go through the directions on the PyBites website to connect your GitHub account to your PyBites account.\n2. Make sure you have Chrome and chromedriver installed and on `$PATH`.\n3. Create environment variables for your PyBites username and password (`PYBITES_USERNAME` and `PYBITES_PASSWORD` respectively). If you signed up for PyBites by authenticating through GitHub or Google, you may need to set a password manually in order to use `eatlocal`.\n4. Create an environment variable for you local Pybites repo(`PYBITES_REPO`).\n\n### Install Chrome and Chromedriver\n\n#### macOS\n\nOne option is to use homebrew [homebrew](https://brew.sh/). \n\nInstall chrome:\n\n```bash\nbrew install --cask google-chrome\n```\n\nInstall chromedriver:\n\n```bash\nbrew install chromedriver\n```\n\nBefore you run chromedriver for the first time, you must explicitly give permission since the developer has not been verified. Running the following command in the terminal removes the warning put in place by Apple:\n\n```bash\nxattr -d com.apple.quarantine $(which chromedriver)\n```\n\nHomebrew automatically puts chromedriver on `$PATH` for you. And since homebrew handles both chrome and chromedriver installations for me, I can run `brew update && brew upgrade` to help ensure I have the same version number for both chrome and chromedriver. If you do not go the homebrew route, you must manually ensure that your version of chrome matches the version of chromedriver.\n\n\n#### Linux\n\nUnfortunately, I did not find some fancy package manager for Linux, but I was able to install chrome and chromedriver manually for Linux Mint.\n\nNavigate to the download page for [google chrome](https://www.google.com/chrome/) and download the appropriate version for your system. Then, open up a terminal and navigate to where you downloaded the file. For me it was `~/Downloads`. I ran the following commands to install and check which version I have.\n\n```bash\ncd ~/Downloads\nsudo dpkg -i google-chrome-stable_current_amd64.deb\ngoogle-chrome --version\n```\n\nNext, navigate to the [chromedriver download page](https://chromedriver.chromium.org/downloads) and choose the version that matches the output from `google-chrome --version`. Download that file that matches your system. Head back to your terminal.\n\n1. Ensure that you have unzip installed:\n\n```bash\nsudo apt install unzip\n```\n\n2. Unzip the chromedriver file. For me it was located in the downloads folder: \n\n```bash\nunzip ~/Downloads/chromedriver_linux64.zip -d ~/Downloads\n```\n\n3. Make it executable and move to `/usr/local/share`:\n\n```bash\nchmod +x ~/Downloads/chromedriver\nsudo mv -f ~/Downloads/chromedriver /usr/local/share/chromedriver\n```\n\n4. Create symlinks:\n\n```bash\nsudo ln -s /usr/local/share/chromedriver /usr/local/bin/chromedriver\n```\n5. Confirm you have access:\n\n```bash\nwhich chromedriver\n```\n\n#### Windows\n\nIf working in windows powershell you can use [chocolately](https://chocolatey.org/) to install chromedriver.\n\nI\'ve found that in order to install packages I have to use an elevated administrative shell, with `choco install chromedriver`.\n\nI attempted to use `eatlocal` from [WSL2](https://docs.microsoft.com/en-us/windows/wsl/about) but there seems to be an issue with `google-chrome` itself. I could not get it to work.\n\n### PyBites Credentials and Local Repo\n\nYou must have your PyBites username and password stored in the environment variables `PYBITES_USERNAME` and `PYBITES_PASSWORD` respectively. Your local PyBites Repo should be stored in the environment variable `PYBITES_REPO`.\n\nIf you have cloned the repo, you can use the conveniently provided `.env-template` to store your credentials. Copy the template and save as `.env`. Then set your username and password on two separate lines. \n\n```bash\n# set username and password\nPYBITES_USERNAME=<username>\nPYBITES_PASSWORD=<password>\nPYBITES_REPO=</path/to/local/repo>\n```\n\n#### macOS/Linux\n\nThere are two methods to handle this in.\n\n**Virtual Environment Method**\n\nA note of warning: If you use this method make sure that your virtual environment is not being pushed to GitHub. If you accidentally push your virtual environment—clearly that has never happened to me—then you have exposed your password and should change it immediately.\n\n1. Create a virtual environment for your PyBites repo:\n\n```bash\npython3 -m venv .venv\n```\n\n2. Add the line `.venv` to your `.gitignore` file.\n\n```bash\necho ".venv" >> .gitignore\n```\n\n3. With the environment deactivated, use your favorite text editor (I use nvim, btw) to open the activate file, e.g., `nvim .venv/bin/activate` and add the following lines:\n\n```bash\nexport PYBITES_USERNAME=<username>\nexport PYBITES_PASSWORD=<password>\nexport PYBITES_REPO=</path/to/local/repo>\n```\n\n4. Activate the environment `source .venv/bin/activate`.\n\n**Shell RC Method**\n\nIf you are not using a virtual environment, you can add the variables directly to your shell config. \n\n1. I use zsh. So I would use my favorite text editor `nvim ~/.zshrc` and set the variables by adding the same three lines as above:\n\n```bash\nexport PYBITES_USERNAME=<username>\nexport PYBITES_PASSWORD=<password>\nexport PYBITES_REPO=</path/to/local/repo>\n```\n\n2. Either exit your terminal completely and reopen, or source your config file with `source ~/.zshrc`.\n\n#### Windows\n\nI don\'t know of a way to do this other than graphically (Booo!). If you like pictures follow this [tutorial](https://windowsloop.com/add-environment-variable-in-windows-10).\n\n1. Open the Start menu by pressing the “Windows Key”.\n2. Type “Environment variables” and click on the “Edit the system environment variables” result.\n3. Click on the "Advanced" tab.\n4. Click "Environment Variables".\n5. Under "User variables" click "New".\n6. In the "Variable name" field enter: PYBITES_USERNAME\n7. In the "Variable value" field enter: <username>\n8. Repeat steps 5-7 for the password variable.\n9. Repeat steps 5-7 for the repo variable.\n10. Click "Ok"\n11. Click "Apply"\n12. Restart your computer.\n',
    'author': 'Russell Helmstedter',
    'author_email': 'rhelmstedter@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/rhelmstedter/eatlocal',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from pathlib import Path
from typing import Union

from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys

from .constants import LOGIN_URL


def driver_setup(path: Union[str, Path] = None) -> webdriver.Chrome:
    """Configures a headless Chrome wedriver and returns it.

    If a path is given, it's used to set the driver's default download
    directory.

    :path: Union[str, Path]
    :returns: configured webdriver.Chrome
    """

    path = str(Path(path or Path.cwd()).resolve())

    options = Options()
    options.add_argument("--headless")
    options.add_argument("window-size=1920x1080")
    chrome_prefs = {"download.default_directory": path}
    options.experimental_options["prefs"] = chrome_prefs

    return webdriver.Chrome(options=options)


def pybites_login(
    driver: webdriver.Chrome, username: str, password: str, verbose: bool = False
) -> None:
    """Authenticate this driver instance with the given credentials.

    :driver: webdriver.Chrome
    :username: str
    :password: str
    :returns: None
    """

    if verbose:
        print("Logging into PyBites")
    driver.get(LOGIN_URL)

    username_field = driver.find_element(By.ID, "id_username")
    username_field.send_keys(username)
    password_field = driver.find_element(By.ID, "id_password")
    password_field.send_keys(password)
    password_field.send_keys(Keys.RETURN)
